@@ -41,6 +41,7 @@
   #:use-module (cv support push)
   #:use-module (cv support keyword)
   #:use-module (cv support modules)
+  #:use-module (cv support s32vector)
   #:use-module (cv support f32vector)
   #:use-module (cv support libguile-cv)
   #:use-module (cv support utils)
@@ -49,13 +50,14 @@
   #:use-module (cv idata)
   #:use-module (cv impex)
   #:use-module (cv imgproc)
-  #:use-module (cv filters)
-  #:use-module (cv morphology)
-  #:use-module (cv segmentation)
-  #:use-module (cv features)
   #:use-module (cv adds)
   #:use-module (cv compose)
   #:use-module (cv histogram)
+  #:use-module (cv features)
+  #:use-module (cv texture)
+  #:use-module (cv filters)
+  #:use-module (cv morphology)
+  #:use-module (cv segmentation)
   #:use-module (cv utils)
 
   #:duplicates (merge-generics
@@ -80,7 +82,8 @@
 			      (cv support push)
 			      (cv support keyword)
 			      (cv support modules)
-			      (cv support f32vector)
+			      (cv support s32vector)
+                              (cv support f32vector)
                               (cv support libguile-cv)
                               (cv support utils)
                               (cv support latex)
@@ -88,11 +91,12 @@
 			      (cv idata)
 			      (cv impex)
 			      (cv imgproc)
+                              (cv adds)
+                              (cv compose)
+                              (cv histogram)
+                              (cv features)
+                              (cv texture)
 			      (cv filters)
 			      (cv morphology)
 			      (cv segmentation)
-                              (cv features)
-			      (cv adds)
-                              (cv compose)
-                              (cv histogram)
 			      (cv utils)))
