@@ -56,8 +56,13 @@
 	    im-or
             im-xor
 	    im-complement
+            im-range
 	    im-min
 	    im-max
+            im-map
+            im-map-channel
+            im-reduce
+            im-reduce-channel
             im-inverse
             im-inverse-channel
             im-transpose
@@ -201,24 +206,6 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
 		  (f32vector-set! c-copy i 255.0))))))))
       (error "Invalid threshold: " threshold)))
 
-(define (im-scalar-op image val op)
-  (match image
-    ((width height n-chan idata)
-     (list width height n-chan
-           (let ((map-proc (if (and (> n-chan 1)
-                                    (%use-par-map)) par-map map)))
-	     (map-proc (lambda (channel)
-			 (im-scalar-channel-op channel width height val op))
-                       idata))))))
-
-(define (im-scalar-channel-op channel width height val op)
-  (let ((to (im-make-channel width height))
-        (n-cell (* width height)))
-    (do ((i 0
-	    (+ i 1)))
-	((= i n-cell) to)
-      (f32vector-set! to i (op (f32vector-ref channel i) val)))))
-
 (define (im-matrix-op image img-2 op)
   (match image
     ((width height n-chan idata)
@@ -246,6 +233,13 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
     (f32vector-set! to i (op (f32vector-ref channel i)
                              (f32vector-ref channel-2 i))))))
 
+(define (matrix-multiply c1 width-1 height-1 c2 width-2)
+  (f32vector-matrix-multiply c1 width-1 height-1 c2 width-2))
+
+(define (matrix-divide c1 width-1 height-1 c2 width-2)
+  (f32vector-matrix-multiply c1 width-1 height-1
+                             (f32vector-inverse c2) width-2))
+
 (define (im-matrix-multdiv-op img-1 img-2 op)
   ;; The product is defined only if the number of columns in img-1 is
   ;; equal to the number of rows in img-2.
@@ -266,150 +260,127 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
             (error "Size missmatch.")))))))
 
 
-
 (define-method (im-add image (val <number>))
-  (im-scalar-op image val +))
+  (im-map (lambda (p-val) (+ p-val val)) image))
 
 (define-method (im-add-channel channel width height (val <number>))
-  (im-scalar-channel-op channel width height val +))
+  (im-map-channel (lambda (p-val) (+ p-val val)) width height channel))
 
-(define-method (im-add image (img-2 <list>))
-  (im-matrix-op image img-2 +))
+(define-method (im-add . images)
+  (apply im-map + images))
 
-(define-method (im-add-channel channel width height (channel-2 <uvec>))
-  (im-matrix-channel-op channel width height channel-2 +))
+(define-method (im-add-channel width height . channels)
+  (apply im-map-channel + width height channels))
 
 
 (define-method (im-subtract image (val <number>))
-  (im-scalar-op image val -))
+  (im-map (lambda (p-val) (- p-val val)) image))
 
 (define-method (im-subtract-channel channel width height (val <number>))
-  (im-scalar-channel-op channel width height val -))
+  (im-map-channel (lambda (p-val) (- p-val val)) width height channel))
 
-(define-method (im-subtract image (img-2 <list>))
-  (im-matrix-op image img-2 -))
+(define-method (im-subtract . images)
+  (apply im-map - images))
 
-(define-method (im-subtract-channel channel width height (channel-2 <uvec>))
-  (im-matrix-channel-op channel width height channel-2 -))
+(define-method (im-subtract-channel width height . channels)
+  (apply im-map-channel - width height channels))
 
 
 (define-method (im-multiply image (val <number>))
-  (im-scalar-op image val *))
+  (im-map (lambda (p-val) (* p-val val)) image))
 
 (define-method (im-multiply-channel channel width height (val <number>))
-  (im-scalar-channel-op channel width height val *))
+  (im-map-channel (lambda (p-val) (* p-val val)) width height channel))
 
-(define-method (im-multiply img-1 (img-2 <list>))
-  (im-matrix-multdiv-op img-1 img-2 im-multiply-channel))
+(define (im-multiply-1 prev images)
+  (if (null? images)
+      prev
+      (match images
+        ((ii . rest)
+         (im-multiply-1 (im-matrix-multdiv-op prev ii matrix-multiply)
+                        rest)))))
 
-(define-method (im-multiply-channel c1 width-1 height-1 (c2 <uvec>) width-2)
-  (f32vector-matrix-multiply c1 width-1 height-1 c2 width-2))
+(define-method (im-multiply . images)
+  (match images
+    ((i1 . rest)
+     (im-multiply-1 i1 rest))
+    (else
+     (error "Wrong arguments:" images))))
+
+(define (im-multiply-channel-1 channel width height rest)
+  (if (null? rest)
+      (values channel width height)
+      (match rest
+        ((channel-i width-i height-i . rest)
+         (im-multiply-channel-1 (f32vector-matrix-multiply channel width height
+                                                           channel-i width-i)
+                                width-i height rest)))))
+
+(define-method (im-multiply-channel . rest)
+  (match rest
+    ((channel width height . rest)
+     (im-multiply-channel-1 channel width height rest))
+    (else
+     (error "Wrong arguments:" rest))))
 
 
 (define-method (im-divide image (val <number>))
-  (im-scalar-op image val /))
+  (im-map (lambda (p-val) (/ p-val val)) image))
 
 (define-method (im-divide-channel channel width height (val <number>))
-  (im-scalar-channel-op channel width height val /))
-
-(define-method (im-divide img-1 (img-2 <list>))
-  (im-matrix-multdiv-op img-1 img-2 im-divide-channel))
-
-(define-method (im-divide-channel c1 width-1 height-1 (c2 <uvec>) width-2)
-  (f32vector-matrix-multiply c1 width-1 height-1 (f32vector-inverse c2) width-2))
+  (im-map-channel (lambda (p-val) (/ p-val val)) width height channel))
 
 
-(define (im-and . images)
+(define (im-divide-1 prev images)
+  (if (null? images)
+      prev
+      (match images
+        ((ii . rest)
+         (im-divide-1 (im-matrix-multdiv-op prev ii matrix-divide)
+                        rest)))))
+
+(define-method (im-divide . images)
   (match images
-    ((image . rest)
-     (match image
-       ((width height n-chan _)
-	(if (and (apply = (cons width (im-collect rest 'width)))
-		 (apply = (cons height (im-collect rest 'height)))
-                 (apply = (cons n-chan (im-collect rest 'n-channel))))
-	    (let ((img-2 (im-copy image))
-		  (n-cell (* width height))
-		  (c-channels (im-collect (map im-rgb->gray rest) 'gray)))
-	      (list width height n-chan
-                    (let ((map-proc (if (and (> n-chan 1)
-                                             (%use-par-map)) par-map map)))
-		      (map-proc
-		       (lambda (channel)
-			 (do ((i 0
-				 (+ i 1)))
-			     ((= i n-cell) channel)
-			   (unless (f32vector-and-at-offset c-channels i)
-			     (f32vector-set! channel i 0.0))))
-		       (match img-2 ((_ _ _ idata) idata))))))
-	    (error "Size mismatch.")))))
-    ((image) image)
-    (() (error "Invalid argument: " images))))
+    ((i1 . rest)
+     (im-divide-1 i1 rest))
+    (else
+     (error "Wrong arguments:" images))))
 
-(define (im-or . images)
-  (match images
-    ((image . rest)
-     (match image
-       ((width height n-chan _)
-	(if (and (apply = (cons width (im-collect rest 'width)))
-		 (apply = (cons height (im-collect rest 'height)))
-                 (apply = (cons n-chan (im-collect rest 'n-channel))))
-	    (let ((img-2 (im-copy image))
-		  (n-cell (* width height))
-		  (c-channels (im-collect (map im-rgb->gray rest) 'gray)))
-	      (list width height n-chan
-                    (let ((map-proc (if (and (> n-chan 1)
-                                             (%use-par-map)) par-map map)))
-		      (map-proc
-		       (lambda (channel)
-			 (do ((i 0
-				 (+ i 1)))
-			     ((= i n-cell) channel)
-			   (unless (> (f32vector-ref channel i) 0.0)
-			     (f32vector-set! channel i
-					     (f32vector-mean-at-offset c-channels i)))))
-		       (match img-2 ((_ _ _ idata) idata))))))
-	    (error "Size mismatch.")))))
-    ((image) image)
-    (() (error "Invalid argument: " images))))
+(define (im-divide-channel-1 channel width height rest)
+  (if (null? rest)
+      (values channel width height)
+      (match rest
+        ((channel-i width-i height-i . rest)
+         (im-divide-channel-1 (f32vector-matrix-multiply channel width height
+                                                         (f32vector-inverse channel-i)
+                                                         width-i)
+                                width-i height rest)))))
 
-(define (im-xor . images)
-  (match images
-    ((image . rest)
-     (match image
-       ((width height n-chan _)
-	(if (and (apply = (cons width (im-collect rest 'width)))
-		 (apply = (cons height (im-collect rest 'height)))
-                 (apply = (cons n-chan (im-collect rest 'n-channel))))
-            (list width height n-chan
-                  (let ((map-proc (if (and (> n-chan 1)
-                                           (%use-par-map)) par-map map)))
-                    (map-proc (lambda (channels)
-                                (im-xor-channels channels width height))
-                        (apply zip (im-collect images 'channels)))))
-	    (error "Size mismatch.")))))
-    ((image) image)
-    (() (error "Invalid argument: " images))))
+(define-method (im-divide-channel . rest)
+  (match rest
+    ((channel width height . rest)
+     (im-divide-channel-1 channel width height rest))
+    (else
+     (error "Wrong arguments:" rest))))
 
-(define (im-xor-channels channels width height)
-  (let ((to (im-make-channel width height))
-        (n-cell (* width height)))
-    (do ((i 0
-            (+ i 1)))
-        ((= i n-cell))
-      (f32vector-set! to i
-                      (f32vector-xor-at-offset channels i)))
-    to))
 
-(define (im-complement image)
+(define (im-range image)
   (match image
     ((width height n-chan idata)
-     (list width height n-chan
-           (let ((map-proc (if (and (> n-chan 1)
-                                    (%use-par-map)) par-map map)))
-	     (map-proc
-	      (lambda (channel)
-		(f32vector-complement channel))
-	      idata))))))
+     (match idata
+       ((c)
+        (im-range-channel c width))
+       (else
+        (let ((map-proc (if (%use-par-map) par-map map)))
+          (map-proc (lambda (channel)
+                      (im-range-channel channel width))
+              idata)))))))
+
+(define (im-range-channel channel width)
+  (match (f32vector-range channel)
+    ((mini p-mini maxi p-maxi)
+     (list mini (quotient p-mini width) (remainder p-mini width)
+           maxi (quotient p-maxi width) (remainder p-maxi width)))))
 
 (define (im-min image)
   (match image
@@ -449,6 +420,159 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
       (f32vector-max channel)
     (values val (quotient pos width) (remainder pos width))))
 
+(define (im-reduce image proc default)
+  (match image
+    ((width height n-chan idata)
+     (match idata
+       ((c)
+        (im-reduce-channel c proc default))
+       (else
+        (let ((map-proc (if (%use-par-map) par-map map)))
+          (map-proc (lambda (channel)
+                      (im-reduce-channel channel proc default))
+              idata)))))))
+
+(define (im-reduce-channel channel proc default)
+  (f32vector-reduce channel proc default))
+
+(define (im-map proc . images)
+  (match images
+    ((image . rest)
+     (match image
+       ((width height n-chan _)
+	(if (and (apply = (apply im-collect 'width rest))
+		  (apply = (apply im-collect 'height rest))
+		  (apply = (apply im-collect 'n-channel rest)))
+            (list width height n-chan
+                  (let ((map-proc (if (and (> n-chan 1)
+                                           (%use-par-map)) par-map map)))
+                    (map-proc (lambda (channels)
+                                (apply im-map-channel proc width height channels))
+                        (apply zip (apply im-collect 'channels images)))))
+	    (error "Size mismatch.")))))
+    ((image)
+     (match image
+       ((width height n-chan idata)
+        (list width height n-chan
+              (let ((map-proc (if (and (> n-chan 1)
+                                       (%use-par-map)) par-map map)))
+                (map-proc (lambda (channel)
+                            (im-map-channel proc width height channel))
+                    idata))))))
+    (()
+     (error "Invalid argument: " images))))
+
+(define (im-map-channel proc width height . channels)
+  (let ((to (im-make-channel width height))
+        (n-cell (* width height))
+        (n-channels (length channels)))
+    (if (> n-channels 1)
+        (do ((i 0
+                (+ i 1)))
+            ((= i n-cell))
+          (f32vector-set! to i
+                          (apply proc
+                                 (f32vector-ref-at-offset channels i))))
+        (do ((c (car channels))
+             (i 0
+                (+ i 1)))
+            ((= i n-cell))
+          (f32vector-set! to i
+                            (proc (f32vector-ref c i)))))
+    to))
+
+(define (im-and . images)
+  (match images
+    ((image . rest)
+     (match image
+       ((width height n-chan _)
+	(if (and (apply = (apply im-collect 'width rest))
+		  (apply = (apply im-collect 'height rest))
+		  (apply = (apply im-collect 'n-channel rest)))
+	    (let ((img-2 (im-copy image))
+		  (n-cell (* width height))
+		  (c-channels (apply im-collect 'gray (map im-rgb->gray rest))))
+	      (list width height n-chan
+                    (let ((map-proc (if (and (> n-chan 1)
+                                             (%use-par-map)) par-map map)))
+		      (map-proc
+		       (lambda (channel)
+			 (do ((i 0
+				 (+ i 1)))
+			     ((= i n-cell) channel)
+			   (unless (f32vector-and-at-offset c-channels i)
+			     (f32vector-set! channel i 0.0))))
+		       (match img-2 ((_ _ _ idata) idata))))))
+	    (error "Size mismatch.")))))
+    ((image) image)
+    (() (error "Invalid argument: " images))))
+
+(define (im-or . images)
+  (match images
+    ((image . rest)
+     (match image
+       ((width height n-chan _)
+	(if (and (apply = (apply im-collect 'width rest))
+		  (apply = (apply im-collect 'height rest))
+		  (apply = (apply im-collect 'n-channel rest)))
+	    (let ((img-2 (im-copy image))
+		  (n-cell (* width height))
+		  (c-channels (apply im-collect 'gray (map im-rgb->gray rest))))
+	      (list width height n-chan
+                    (let ((map-proc (if (and (> n-chan 1)
+                                             (%use-par-map)) par-map map)))
+		      (map-proc
+		       (lambda (channel)
+			 (do ((i 0
+				 (+ i 1)))
+			     ((= i n-cell) channel)
+			   (unless (> (f32vector-ref channel i) 0.0)
+			     (f32vector-set! channel i
+					     (f32vector-mean-at-offset c-channels i)))))
+		       (match img-2 ((_ _ _ idata) idata))))))
+	    (error "Size mismatch.")))))
+    ((image) image)
+    (() (error "Invalid argument: " images))))
+
+(define (im-xor . images)
+  (match images
+    ((image . rest)
+     (match image
+       ((width height n-chan _)
+	(if  (and (apply = (apply im-collect 'width rest))
+		  (apply = (apply im-collect 'height rest))
+		  (apply = (apply im-collect 'n-channel rest)))
+            (list width height n-chan
+                  (let ((map-proc (if (and (> n-chan 1)
+                                           (%use-par-map)) par-map map)))
+                    (map-proc (lambda (channels)
+                                (im-xor-channels channels width height))
+                        (apply zip (apply im-collect 'channels images)))))
+	    (error "Size mismatch.")))))
+    ((image) image)
+    (() (error "Invalid argument: " images))))
+
+(define (im-xor-channels channels width height)
+  (let ((to (im-make-channel width height))
+        (n-cell (* width height)))
+    (do ((i 0
+            (+ i 1)))
+        ((= i n-cell))
+      (f32vector-set! to i
+                      (f32vector-xor-at-offset channels i)))
+    to))
+
+(define (im-complement image)
+  (match image
+    ((width height n-chan idata)
+     (list width height n-chan
+           (let ((map-proc (if (and (> n-chan 1)
+                                    (%use-par-map)) par-map map)))
+	     (map-proc
+	      (lambda (channel)
+		(f32vector-complement channel))
+	      idata))))))
+
 (define (im-transpose image)
   (match image
     ((width height n-chan idata)
@@ -485,24 +609,24 @@ Target.B = ((1 - Source.A) * BGColor.B) + (Source.A * Source.B)
 (define (im-inverse-channel channel width height)
   (f32vector-inverse channel))
 
-(define (im-normalize image)
+(define* (im-normalize image #:key (val 255.0))
   (match image
     ((width height n-chan idata)
-     (list height width n-chan
+     (list width height n-chan
            (let ((map-proc (if (and (> n-chan 1)
                                     (%use-par-map)) par-map map)))
 	     (map-proc (lambda (channel)
-			 (im-normalize-channel channel width height))
+			 (im-normalize-channel channel width height #:val val))
 	       idata))))))
 
-(define (im-normalize-channel channel width height)
+(define* (im-normalize-channel channel width height #:key (val 255.0))
   (let ((to (im-make-channel width height))
         (n-cell (* width height)))
     (do ((i 0
 	    (+ i 1)))
 	((= i n-cell))
       (f32vector-set! to i (/ (f32vector-ref channel i)
-                              255)))
+                              val)))
     to))
 
 (define* (im-scrap image val #:key (pred <) (con 8) (bg 'black))
