@@ -35,6 +35,7 @@
   #:use-module (cv support push)
   #:use-module (cv support keyword)
   #:use-module (cv support modules)
+  #:use-module (cv support s32vector)
   #:use-module (cv support f32vector)
   #:use-module (cv support libguile-cv)
   #:use-module (cv support utils)
@@ -56,6 +57,7 @@
 			      (cv support push)
 			      (cv support keyword)
 			      (cv support modules)
+                              (cv support s32vector)
 			      (cv support f32vector)
                               (cv support libguile-cv)
                               (cv support utils)
