@@ -36,13 +36,15 @@
   #:use-module (cv support float)
   #:use-module (cv support libguile-cv)
 
-  #:export (f32vector-min
+  #:export (f32vector-range
+            f32vector-min
 	    f32vector-max
             f32vector-reduce
             f32vector-mean
             f32vector-std-dev
 	    f32vector-copy
 	    f32vector-complement
+            f32vector-ref-at-offset
 	    f32vector-and-at-offset
 	    f32vector-or-at-offset
             f32vector-xor-at-offset
@@ -55,51 +57,55 @@
 	    f32vector-index
 	    f32vector-count-distinct
             f32vector-inverse
-            f32vector-matrix-multiply))
+            f32vector-matrix-multiply
+            f32vector->s32vector))
 
 
-(define* (f32vector-min v #:key (prec 1.0e-4))
+(define* (f32vector-range v #:key (prec 1.0e-4))
   (let ((n-cell (f32vector-length v)))
     (case n-cell
       ((0) (error "Empty vector: " v))
       (else
        (do ((mini (f32vector-ref v 0))
-            (pos 0)
+            (maxi (f32vector-ref v 0))
+            (p-mini 0)
+            (p-maxi 0)
             (i 1
                (+ i 1)))
-           ((= i n-cell) (values mini pos))
+           ((= i n-cell)
+            (list mini p-mini maxi p-maxi))
          (let ((val (f32vector-ref v i)))
            (when (float<? val mini prec)
              (set! mini val)
-             (set! pos i))))))))
-
-(define* (f32vector-max v #:key (prec 1.0e-4))
-  (let ((n-cell (f32vector-length v)))
-    (case n-cell
-      ((0) (error "Empty vector: " v))
-      (else
-       (do ((maxi (f32vector-ref v 0))
-            (pos 0)
-            (i 1
-               (+ i 1)))
-           ((= i n-cell) (values maxi pos))
-         (let ((val (f32vector-ref v i)))
+             (set! p-mini i))
            (when (float>? val maxi)
              (set! maxi val)
-             (set! pos i))))))))
+             (set! p-maxi i))))))))
 
-(define* (f32vector-reduce init op v #:key (n-cell #f))
+(define* (f32vector-min v #:key (prec 1.0e-4))
+  (match (f32vector-range v #:prec prec)
+    ((mini p-mini maxi p-maxi)
+     (values mini p-mini))))
+
+(define* (f32vector-max v #:key (prec 1.0e-4))
+  (match (f32vector-range v #:prec prec)
+    ((mini p-mini maxi p-maxi)
+     (values maxi p-maxi))))
+
+(define* (f32vector-reduce v proc default #:key (n-cell #f))
   (let ((n-cell (or n-cell
-                    (f32vector-length v)))
-        (result init))
-    (do ((i 0
-            (+ i 1)))
-        ((= i n-cell) result)
-      (set! result (op result (f32vector-ref v i))))))
+                    (f32vector-length v))))
+    (if (= n-cell 0)
+        default
+        (do ((i 1
+                (+ i 1))
+             (prev (f32vector-ref v 0)
+                   (proc (f32vector-ref v i) prev)))
+            ((= i n-cell) prev)))))
 
 (define* (f32vector-mean v #:key (n-cell #f))
   (let ((n-cell (f32vector-length v)))
-    (/ (f32vector-reduce 0 + v #:n-cell n-cell) n-cell)))
+    (/ (f32vector-reduce v + 0 #:n-cell n-cell) n-cell)))
 
 (define* (f32vector-std-dev v #:key (n-cell #f) (mean #f))
   (let* ((n-cell (or n-cell (f32vector-length v)))
@@ -145,6 +151,11 @@
       (f32vector-set! copy i
 		      (- of (f32vector-ref vector i))))))
 
+(define (f32vector-ref-at-offset vectors i)
+  (map (lambda (vector)
+         (f32vector-ref vector i))
+    vectors))
+
 (define* (f32vector-and-at-offset vectors i
 				  #:key (prec 1.0e-4))
   (let ((n-vec (length vectors))
@@ -175,7 +186,7 @@
     ((a . rests)
      (pixel-logior-1 rests a))
     (else
-     (error "Invalid argument: vals"))))
+     (error "Invalid argument:" vals))))
 
 (define (f32vector-xor-at-offset vectors i)
   (pixel-logior (map (lambda (vector)
@@ -185,9 +196,7 @@
 (define (f32vector-sum-at-offset vectors i)
   (reduce +
 	  0.0
-	  (map (lambda (vector)
-		 (f32vector-ref vector i))
-	    vectors)))
+          (f32vector-ref-at-offset vectors i)))
 
 (define (f32vector-mean-at-offset vectors i)
   (let ((n-vec (length vectors)))
@@ -328,7 +337,7 @@
              (k 0
                 (+ k 1)))
             ((= k m)
-             (f32vector-set! ab (+ (* i n) j) sub))
+             (f32vector-set! ab (+ (* i p) j) sub))
           (set! sub
                 (+ sub (* (f32vector-ref a (+ (* i m) k))
                           (f32vector-ref b (+ (* k p) j))))))))))
@@ -342,6 +351,15 @@
       (f32vector-set! to i
                       (expt (f32vector-ref v i) -1)))))
 
+(define (f32vector->s32vector v)
+  (let* ((n-cell (f32vector-length v))
+	 (to (make-s32vector n-cell)))
+    (do ((i 0
+	    (+ i 1)))
+	((= i n-cell) to)
+      (s32vector-set! to i
+                      (float->int (f32vector-ref v i))))))
+
 
 ;;;
 ;;; Vigra_c bindings
