@@ -159,20 +159,20 @@
 	 (img-6 (im-make 3 2 4))
 	 (list-1 (list img-1 img-2))
 	 (list-2 (list img-1 img-5)))
-    (assert-true (im-collect list-1 'size))
-    (assert-true (im-collect list-1 'width))
-    (assert-true (im-collect list-1 'height))
-    (assert-true (im-collect list-1 'n-channel))
-    (assert-true (im-collect list-1 'channels))
-    (assert-true (im-collect list-1 'chan-0))
-    (assert-true (im-collect list-1 'gray))
-    (assert-true (im-collect list-1 'red))
-    (assert-true (im-collect list-1 'chan-1))
-    (assert-true (im-collect list-1 'green))
-    (assert-true (im-collect list-1 'chan-2))
-    (assert-true (im-collect list-1 'blue))
-    (assert-true (im-collect list-1 'blue))
-    (assert-true (im-collect (list img-6) 'chan-3))))
+    (assert-true (apply im-collect 'size list-1))
+    (assert-true (apply im-collect 'width list-1))
+    (assert-true (apply im-collect 'height list-1))
+    (assert-true (apply im-collect 'n-channel list-1))
+    (assert-true (apply im-collect 'channels list-1))
+    (assert-true (apply im-collect 'chan-0 list-1))
+    (assert-true (apply im-collect 'gray list-1))
+    (assert-true (apply im-collect 'red list-1))
+    (assert-true (apply im-collect 'chan-1 list-1))
+    (assert-true (apply im-collect 'green list-1))
+    (assert-true (apply im-collect 'chan-2 list-1))
+    (assert-true (apply im-collect 'blue list-1))
+    (assert-true (apply im-collect 'blue list-1))
+    (assert-true (im-collect 'chan-3 img-6))))
 
 
 (define-method (test-idata-3 (self <guile-cv-tests-cv>))
@@ -211,20 +211,18 @@
       (assert-equal '(85 95 3) (im-size bf))
       (assert ((@@ (cv impex) vigra-save-rgb-image) bf t-file 85 95)))))
 
-
-(define-method (test-im-min/max (self <guile-cv-tests-cv>))
-  (let* ((gray (make-test-image-gray 3 2))
-         (rgb (make-test-image-rgb 3 2)))
-    (receive (val row col)
-        (im-min gray)
-      (assert-numeric-= val 0.0 1.0e-4)
-      (assert-numeric-= row 0 1.0e-4)
-      (assert-numeric-= col 0 1.0e-4))
-    (receive (val row col)
-        (im-max gray)
-      (assert-numeric-= val 5.0 1.0e-4)
-      (assert-numeric-= row 1 1.0e-4)
-      (assert-numeric-= col 2 1.0e-4))
+(define-method (test-im-range (self <guile-cv-tests-cv>))
+  (let ((gray (make-test-image-gray 3 2))
+        (rgb (make-test-image-rgb 3 2))
+        (prec 1.0e-4))
+    (match (im-range gray)
+      ((mini i-mini j-mini maxi i-maxi j-maxi)
+       (assert-numeric-= mini 0.0 prec)
+       (assert-numeric-= i-mini 0 prec)
+       (assert-numeric-= j-mini 0 prec)
+       (assert-numeric-= maxi 5.0 prec)
+       (assert-numeric-= i-maxi 1 prec)
+       (assert-numeric-= j-maxi 2 prec)))
     (let ((rgb-mins (im-min rgb))
           (expected-mins '((0.0 0 0) (6.0 0 0) (12.0 0 0)))
           (rgb-maxs (im-max rgb))
@@ -248,7 +246,6 @@
                        (zip a b)))))
           (zip rgb-maxs expected-maxs)))))
 
-
 (define-method (test-im-and (self <guile-cv-tests-cv>))
   (let* ((empty (make-test-image-gray-type 'empty))
 	 (center (make-test-image-gray-type 'center))
@@ -300,62 +297,74 @@
       (assert-true (= (inexact->exact (f32vector-ref di-chan i))
                       (list-ref di-check i))))))
 
-(define (make-add-2-a)
+(define %a
   `(2 3 1 (,#f32(1.0 2.0 3.0 4.0 5.0 6.0))))
 
-(define (make-add-2-b)
+(define %b
   `(2 3 1 (,#f32(2.0 3.0 4.0 5.0 6.0 7.0))))
 
-(define (make-add-2-c)
+(define %c
   `(2 3 1 (,#f32(2.0 4.0 6.0 8.0 10.0 12.0))))
 
-(define (make-add-2-a+b)
+(define %a+b
   `(2 3 1 (,#f32(3.0 5.0 7.0 9.0 11.0 13.0))))
 
-(define (make-add-2-a-b)
+(define %a+b+c
+  `(2 3 1 (,#f32(5.0 9.0 13.0 17.0 21.0 25.0))))
+
+(define %a-b
   `(2 3 1 (,#f32(-1.0 -1.0 -1.0 -1.0 -1.0 -1.0))))
 
-(define (make-add-2-a')
+(define %a-b-c
+  `(2 3 1 (,#f32(-3.0 -5.0 -7.0 -9.0 -11.0 -13.0))))
+
+(define %a'
   `(3 2 1 (,#f32(1.0 3.0 5.0 2.0 4.0 6.0))))
 
-(define (make-add-2-a*a')
+(define %axa'
   `(3 3 1 (,#f32(5.0 11.0 17.0 11.0 25.0 39.0 17.0 39.0 61.0))))
 
+(define %axa'xa
+  `(2 3 1 (,#f32(123.0 156.0 281.0 356.0 439.0 556.0))))
+
 (define-method (test-im-add (self <guile-cv-tests-cv>))
-  (let ((a (make-add-2-a))
-        (b (make-add-2-b))
-        (a+b (make-add-2-a+b)))
-    (assert-true (im-=? (im-add a 1.0) b))
-    (assert-true (im-=? (im-add a b) a+b))))
+  (assert-true (im-=? (im-add %a 1.0) %b))
+  (assert-true (im-=? (im-add %a %b) %a+b))
+  (assert-true (im-=? (im-add %a %b %c) %a+b+c)))
 
 (define-method (test-im-subtract (self <guile-cv-tests-cv>))
-  (let ((a (make-add-2-a))
-        (b (make-add-2-b))
-        (a-b (make-add-2-a-b)))
-    (assert-true (im-=? (im-subtract b 1.0) a))
-    (assert-true (im-=? (im-subtract a b) a-b))))
+  (assert-true (im-=? (im-subtract %b 1.0) %a))
+  (assert-true (im-=? (im-subtract %a %b) %a-b))
+  (assert-true (im-=? (im-subtract %a %b %c) %a-b-c)))
 
 (define-method (test-im-multiply (self <guile-cv-tests-cv>))
-  (let ((a (make-add-2-a))
-        (c (make-add-2-c))
-        (a' (make-add-2-a'))
-        (a*a' (make-add-2-a*a')))
-    (assert-true (im-=? (im-multiply a 2.0) c))
-    (assert-true (im-=? (im-multiply a a') a*a'))))
+  (assert-true (im-=? (im-multiply %a 2.0) %c))
+  (assert-true (im-=? (im-multiply %a %a') %axa'))
+  (assert-true (im-=? (im-multiply %a %a' %a) %axa'xa)))
+
+(define %d1
+  `(2 3 1 (,#f32(2.0 4.0 8.0 16.0 32.0 64.0))))
+
+(define %d2
+  `(2 3 1 (,#f32(1.0 2.0 4.0 8.0 16.0 32.0))))
+
+(define %d3
+  `(3 2 1 (,#f32(2.0 8.0 32.0 4.0 16.0 64.0))))
+
+(define %d4 (im-transpose %d1))
+
+(define %d5
+  `(3 3 1 (,#f32(2.0 0.5 0.125 8.0 2.0 0.5 32.0 8.0 2.0))))
+
+(define %d6 (im-transpose %d5))
+
+(define %d7
+  `(3 3 1 (,#f32(3.0 0.75 0.1875 12.0 3.0 0.75 48.0 12.0 3.0))))
 
 (define-method (test-im-divide (self <guile-cv-tests-cv>))
-  (let* ((a `(2 3 1 (,#f32(2.0 4.0 8.0 16.0 32.0 64.0))))
-         (d (im-inverse
-             (im-transpose
-              `(2 3 1 (,#f32(2.0 4.0 8.0 16.0 32.0 64.0))))))
-         (a/d (im-multiply a d))
-         (a/d-chan (im-channel a/d 0))
-         (a/d-check '(2 1/2 1/8 8 2 1/2 32 8 2)))
-    (do ((i 0
-            (+ i 1)))
-        ((= i 9))
-      (assert-true (= (inexact->exact (f32vector-ref a/d-chan i))
-                      (list-ref a/d-check i))))))
+  (assert-true (im-=? (im-divide %d1 2.0) %d2))
+  (assert-true (im-=? (im-divide %d1 %d4) %d5))
+  (assert-true (im-=? (im-divide %d1 %d4 %d6) %d7)))
 
 (define (make-scrap-test-image)
   (let ((img (im-make 12 12 1)))
@@ -456,5 +465,72 @@
       (assert-exception (im-features img-1 img-2))
       (assert (im-features img-1 l-img)))))
 
+(define %texture-test-image
+  (list 4 4 1
+        (list #f32(0 0 1 1 0 0 1 1 0 2 2 2 2 2 3 3))))
+
+(define  %texture-test-image-2
+  (list 4 4 1
+        (list #f32(0.0  0.0  2.0  2.0
+                   0.0  0.0  2.0  2.0
+                   0.0  4.0  4.0  4.0
+                   4.0  4.0  6.0  6.0))))
+
+(define %glcm-test-result
+  (list 4 4 4
+        (list #f32(2.0  2.0  1.0  0.0
+                   0.0  2.0  0.0  0.0
+                   0.0  0.0  3.0  1.0
+                   0.0  0.0  0.0  1.0)
+              #f32(2.0  1.0  0.0  0.0
+                   0.0  1.0  0.0  0.0
+                   0.0  2.0  2.0  0.0
+                   0.0  0.0  1.0  0.0)
+              #f32(3.0  0.0  0.0  0.0
+                   0.0  2.0  0.0  0.0
+                   2.0  2.0  1.0  0.0
+                   0.0  0.0  2.0  0.0)
+              #f32(1.0  0.0  0.0  0.0
+                   1.0  1.0  0.0  0.0
+                   3.0  1.0  0.0  0.0
+                   0.0  0.0  2.0  0.0))))
+
+(define %glcm-test-result-2
+  (list 4 4 4
+        (list #f32(4.0  4.0  2.0  0.0
+                   0.0  4.0  0.0  0.0
+                   0.0  0.0  6.0  2.0
+                   0.0  0.0  0.0  2.0)
+              #f32(4.0  2.0  0.0  0.0
+                   0.0  2.0  0.0  0.0
+                   0.0  4.0  4.0  0.0
+                   0.0  0.0  2.0  0.0)
+              #f32(6.0  0.0  0.0  0.0
+                   0.0  4.0  0.0  0.0
+                   4.0  4.0  2.0  0.0
+                   0.0  0.0  4.0  0.0)
+              #f32(2.0  0.0  0.0  0.0
+                   2.0  2.0  0.0  0.0
+                   6.0  2.0  0.0  0.0
+                   0.0  0.0  4.0  0.0))))
+
+(define-method (test-im-map (self <guile-cv-tests-cv>))
+  (let ((i1 %texture-test-image)
+        (i2 %texture-test-image-2)
+        (i3 %glcm-test-result)
+        (i4 %glcm-test-result-2))
+    (assert-true (im-=? i1 (im-map identity i1)))
+    (assert-true (im-=? (im-map + i1 i1) i2))
+    (assert-true (im-=? i3 (im-map identity i3)))
+    (assert-true (im-=? (im-map + i3 i3) i4))))
+
+(define-method (test-im-reduce (self <guile-cv-tests-cv>))
+  (assert-numeric-= (im-reduce %texture-test-image + 0) 20.0 1.0e-4)
+  (for-each (lambda (vals)
+              (match vals
+                ((got need)
+                 (assert-numeric-= got need 1.0e-4))))
+      (zip (im-reduce %glcm-test-result + 0)
+           '(12.0 9.0 12.0 9.0))))
 
 (exit-with-summary (run-all-defined-test-cases))
